@@ -233,8 +233,7 @@ hildon_desktop_notification_manager_get_icon (const gchar *icon_name)
 
       if (error)
       {
-        pixbuf = NULL; /* It'd be already NULL */
-	g_warning ("Notification Manager %s:",error->message);
+	g_warning ("Notification icon not loaded: %s", error->message);
         g_error_free (error);
       }
     }
@@ -244,7 +243,7 @@ hildon_desktop_notification_manager_get_icon (const gchar *icon_name)
 
       pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                          icon_name,
-                                         32,
+                                         40,
                                          GTK_ICON_LOOKUP_NO_SVG,
 	                                 &error);
 
@@ -1073,6 +1072,7 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
                                             DBusGMethodInvocation *context)
 {
   GtkTreeIter iter;
+  GdkPixbuf *pixbuf = NULL;
   GHashTable *hints_copy;
 #ifdef HAVE_SQLITE
   GValue *hint;
@@ -1081,7 +1081,19 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
   gboolean valid_actions = TRUE;
   gboolean persistent = FALSE;
   gint i;
-  
+
+  g_return_val_if_fail (*icon != '\0', FALSE);
+  g_return_val_if_fail (summary != '\0', FALSE);
+  g_return_val_if_fail (body != '\0', FALSE);
+
+  pixbuf = hildon_desktop_notification_manager_get_icon (icon);
+
+  if (!pixbuf)
+  {
+    g_warning ("Notification icon not found. Ignoring this notification.");
+    return FALSE;
+  }
+    
 #ifdef HAVE_SQLITE
     hint = g_hash_table_lookup (hints, "persistent");
 
@@ -1130,7 +1142,7 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
 		        HD_NM_COL_APPNAME, app_name,
 		        HD_NM_COL_ID, id,
 		        HD_NM_COL_ICON_NAME, icon,
-		        HD_NM_COL_ICON, hildon_desktop_notification_manager_get_icon (icon),
+		        HD_NM_COL_ICON, pixbuf,
 		        HD_NM_COL_SUMMARY, summary,
 		        HD_NM_COL_BODY, body,
 		        HD_NM_COL_ACTIONS, actions_copy,
@@ -1162,7 +1174,7 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
     gtk_list_store_set (GTK_LIST_STORE (nm),
 		        &iter,
 			HD_NM_COL_ICON_NAME, icon,
-			HD_NM_COL_ICON, hildon_desktop_notification_manager_get_icon (icon),
+			HD_NM_COL_ICON, pixbuf,
 			HD_NM_COL_SUMMARY, summary,
 			HD_NM_COL_BODY, body,
 			HD_NM_COL_REMOVABLE, FALSE,
