@@ -40,6 +40,8 @@
 #define SYSTEM_TRAY_CANCEL_MESSAGE  2
 #endif
 
+#include <libhildonwm/hd-wm.h>
+
 #define STATUSBAR_DEBUG 0
 #if (STATUSBAR_DEBUG)
 #define sb_debug(o,a...) g_debug("sb: " o,##a)
@@ -92,6 +94,7 @@ struct _HildonDesktopPanelExpandablePrivate
   Atom 	      tray_opcode;
   GtkWidget  *invisible;
   gboolean    filter_added;
+  guint	      n_systray_applets;
 #endif
 };
 
@@ -224,6 +227,7 @@ hildon_desktop_panel_expandable_init (HildonDesktopPanelExpandable *panel)
   panel->priv->extension_table = NULL;
 #ifdef SYSTRAY_SUPPORT
   panel->priv->filter_added = FALSE;
+  panel->priv->n_systray_applets = 0;
 #endif
 }
 
@@ -233,7 +237,11 @@ hildon_desktop_panel_expandable_hide_extension (HildonDesktopPanelExpandable *pa
   gdk_pointer_ungrab (GDK_CURRENT_TIME);
   gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 
-  g_object_set (panel->priv->arrow, "active", FALSE, NULL);
+  if (!HILDON_DESKTOP_IS_PANEL_EXPANDABLE (panel))
+    return;
+	  
+  if (panel->priv->arrow && GTK_IS_WIDGET (panel->priv->arrow))
+    g_object_set (panel->priv->arrow, "active", FALSE, NULL);
 
   gtk_grab_remove (GTK_WIDGET (panel->priv->extension_window));
 
@@ -317,6 +325,17 @@ hildon_desktop_panel_expandable_hide_on_delete (GtkWidget *extension_window,
   return TRUE;
 }
 
+static void 
+hildon_desktop_panel_close_fullscreen (HDWM *hdwm, 
+				       gboolean fullscreen, 
+				       HildonDesktopPanelExpandable *panel)
+{
+  if (fullscreen)
+  {
+    hildon_desktop_panel_expandable_hide_extension (panel);	    
+  }	  
+}
+
 static GObject *
 hildon_desktop_panel_expandable_constructor (GType gtype,
 			                     guint n_params, 
@@ -324,6 +343,7 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
 {
   GObject *object;
   HildonDesktopPanelExpandable *panel;
+  GtkWidget *frame;
   gint item_width, item_height;
 
   object = 
@@ -354,7 +374,14 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
 		"column-spacing", 0,
 		NULL);	
 
+  frame = gtk_frame_new (NULL);
+
   gtk_container_add (GTK_CONTAINER (panel->priv->extension_window),
+		     frame);
+
+  gtk_widget_show (frame);
+  
+  gtk_container_add (GTK_CONTAINER (frame),
 		     GTK_WIDGET (panel->priv->extension_table));
 
   gtk_widget_show (GTK_WIDGET (panel->priv->extension_table));
@@ -392,6 +419,14 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
 		          G_CALLBACK (hildon_desktop_panel_expandable_init_systray),
 		          NULL);
 #endif
+
+  HDWM *hdwm = hd_wm_get_singleton ();
+
+  g_signal_connect (hdwm, 
+		    "fullscreen",
+		    G_CALLBACK (hildon_desktop_panel_close_fullscreen),
+		    (gpointer)panel);
+  
   return object;
 }
 
@@ -577,9 +612,20 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
   /* gtk_widget_set_size_request (button, item_width, item_height);*/
   if (GTK_IS_BIN (button) && GTK_IS_WIDGET (GTK_BIN (button)->child))
     gtk_widget_set_size_request (GTK_BIN (button)->child, item_width, item_height);
-  
-  g_object_set (G_OBJECT (item), "position", ex_panel->priv->current_position++, NULL);
-  
+ 
+  if (!STATUSBAR_IS_ITEM_SOCKET (button)) 
+    g_object_set (G_OBJECT (item), "position", ex_panel->priv->current_position++, NULL);
+
+  if (STATUSBAR_IS_ITEM_SOCKET (button))  
+  {	 
+    if (GTK_WIDGET (button)->parent == NULL)
+    {
+      HILDON_DESKTOP_PANEL_CLASS (hildon_desktop_panel_expandable_parent_class)->add_button (panel,button);
+      sb_debug ("adding at the beginnning");
+    }  
+    ex_panel->priv->n_items++;
+  }
+  else 
   if ((ex_panel->priv->n_items+2) > ex_panel->priv->items_p_row && (ex_panel->priv->n_items+1) != ex_panel->priv->items_p_row)
   { 
     if (STATUSBAR_IS_ITEM (button) && !STATUSBAR_ITEM (button)->condition)
@@ -596,7 +642,7 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
       
           HILDON_DESKTOP_PANEL_CLASS (hildon_desktop_panel_expandable_parent_class)->add_button (panel,arrow);
       
-           ex_panel->priv->arrow = arrow;
+          ex_panel->priv->arrow = arrow;
         }
 	
 	hildon_desktop_panel_expandable_add_in_extension (ex_panel, item);
@@ -606,8 +652,6 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
         hildon_desktop_panel_expandable_add_in_extension (ex_panel, item);
       }
     }
-
-     
   }
   else
   {
@@ -662,7 +706,6 @@ hildon_desktop_panel_expandable_add_in_extension (HildonDesktopPanelExpandable *
 
   division = ((guint)((n_items+1) / panel->priv->items_p_row));
 
-g_debug ("fivision is %d and n_items is %d", division, n_items);
   n_rows = division + (((n_items+1) % panel->priv->items_p_row) != 0) ? 1 : 0;
   
   g_object_get (panel->priv->extension_table, "n-rows", &table_rows, NULL);
@@ -766,7 +809,7 @@ hildon_desktop_panel_expandable_arrange_items (HildonDesktopPanelExpandable *pan
       panel->priv->arrow = NULL;
 
     if (!STATUSBAR_IS_ITEM_SOCKET (l->data))
-      gtk_container_parent_remove (GTK_CONTAINER (panel), GTK_WIDGET (l->data));
+      gtk_container_remove (GTK_CONTAINER (panel), GTK_WIDGET (l->data));
   }
 
   for (l = children_table ; l ; l = g_list_next (l))
@@ -858,7 +901,8 @@ hildon_desktop_panel_expandable_add_arrow (HildonDesktopPanelExpandable *panel)
   gtk_widget_show (arrow_button);
 
   g_signal_emit_by_name (panel, "arrow-added", arrow_button);
-		  
+  g_signal_emit_by_name (panel, "queued-button", arrow_button);
+  
   return arrow_button;
 }
 
@@ -1034,8 +1078,10 @@ hildon_desktop_x_event_filter (GdkXEvent *xevent,
       }
 
       g_free (id);
+
+      if (panel->priv->n_systray_applets <= (panel->priv->items_p_row - 1))
+        hildon_desktop_panel_embed_applet (panel,(Window)e->xclient.data.l[2]);
       
-      hildon_desktop_panel_embed_applet (panel,(Window)e->xclient.data.l[2]);
       return GDK_FILTER_CONTINUE;
     }
   }
@@ -1059,6 +1105,8 @@ hildon_desktop_panel_remove_embed (GtkSocket *socket, HildonDesktopPanelExpandab
 
   panel->priv->n_items--;
 
+  panel->priv->n_systray_applets--;
+
   gtk_container_remove (GTK_CONTAINER (panel), parent);
 
   hildon_desktop_panel_expandable_arrange_items (panel);
