@@ -163,7 +163,7 @@ hildon_desktop_get_current_wm_name (HildonDesktopPanelWindowDialog *dialog)
 		      0, 16L, False, XA_WINDOW, &type, &format,
 		      &nitems, &bytes_after, &support_xwin.s);
 
-  if (support_xwin.s == NULL)
+  if (gdk_error_trap_pop () || support_xwin.s == NULL)
       return NULL;
 
   atom_utf8_string = XInternAtom (dpy, "UTF8_STRING", False);
