@@ -205,10 +205,32 @@ hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
 }
 
 static gboolean
-hildon_desktop_popup_menu_paint (GtkWidget      *widget,
-                                 GdkEventExpose *event,
-                                 gpointer        user_data)
+hildon_desktop_popup_menu_paint_viewport (GtkWidget      *widget,
+                                          GdkEventExpose *event,
+                                          gpointer        user_data)
 {
+  static GtkWidgetClass *parent_class = NULL;
+
+  if (!parent_class)
+    parent_class =
+        GTK_WIDGET_CLASS (g_type_class_peek_parent (GTK_VIEWPORT_GET_CLASS (widget)));
+
+  parent_class->expose_event (widget, event);
+
+  return TRUE;
+}
+
+static gboolean
+hildon_desktop_popup_menu_paint_pane (GtkWidget      *widget,
+                                      GdkEventExpose *event,
+                                      gpointer        user_data)
+{
+  static GtkWidgetClass *parent_class = NULL;
+
+  if (!parent_class)
+    parent_class =
+        GTK_WIDGET_CLASS (g_type_class_peek_parent (GTK_VBOX_GET_CLASS (widget)));
+
   gtk_paint_box (widget->style,
                  widget->window,
                  GTK_STATE_NORMAL,
@@ -221,7 +243,9 @@ hildon_desktop_popup_menu_paint (GtkWidget      *widget,
                  widget->allocation.width,
                  widget->allocation.height);
 
-  return FALSE;
+  parent_class->expose_event (widget, event);
+
+  return TRUE;
 }
 
 static GObject *
@@ -330,12 +354,12 @@ hildon_desktop_popup_menu_constructor (GType gtype,
 
   g_signal_connect (menu,
                     "expose_event",
-                    G_CALLBACK (hildon_desktop_popup_menu_paint), 
+                    G_CALLBACK (hildon_desktop_popup_menu_paint_pane), 
                     NULL);
   
   g_signal_connect (menu->priv->viewport,
                     "expose_event",
-                    G_CALLBACK (hildon_desktop_popup_menu_paint), 
+                    G_CALLBACK (hildon_desktop_popup_menu_paint_viewport), 
                     NULL);
   
   return object;
@@ -629,6 +653,22 @@ static gboolean
 hildon_desktop_popup_menu_release_event (GtkWidget      *widget,
                                          GdkEventButton *event)
 {
+  HildonDesktopPopupMenu *menu = HILDON_DESKTOP_POPUP_MENU (widget);
+  gint w,h,x,y;
+
+  if (!menu->priv->selected_item)
+    return TRUE;
+  
+  gtk_widget_get_pointer (GTK_WIDGET (menu->priv->selected_item), &x, &y);
+
+  w = GTK_WIDGET (menu->priv->selected_item)->allocation.width;
+  h = GTK_WIDGET (menu->priv->selected_item)->allocation.height;
+
+  if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+  {
+    gtk_menu_item_activate (menu->priv->selected_item);
+  }
+
   return TRUE;
 } 
 	
@@ -660,8 +700,7 @@ hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
           gtk_item_select (GTK_ITEM (l->data));
           menu->priv->selected_item = GTK_MENU_ITEM (l->data);
         }
-        
-	gtk_menu_item_activate (GTK_MENU_ITEM (l->data));
+
         break;
       }
     }
