@@ -88,6 +88,11 @@ static gboolean hildon_desktop_popup_window_leave_notify (GtkWidget *widget, Gdk
 static gboolean hildon_desktop_popup_window_visibility_notify (GtkWidget          *widget,
 				                               GdkEventVisibility *event,
 			        		               gpointer            data);
+
+static gboolean hildon_desktop_popup_window_pane_delete_event (GtkWidget *widget,
+						  	       GdkEvent  *event,
+							       gpointer   data);
+
 static gboolean hildon_desktop_popup_window_delete_event (GtkWidget *widget,
 							  GdkEvent  *event,
 							  gpointer   data);
@@ -96,13 +101,13 @@ static gboolean hildon_desktop_popup_window_composited_leave_notify (GtkWidget *
 						     		     GdkEventCrossing *event,
 						     		     HildonDesktopPopupWindow *popup);
 
-static gboolean hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
-		                                                       GdkEventButton *event,
-	                                                               HildonDesktopPopupWindow *popup);
+static gboolean hildon_desktop_popup_window_composited_button_press (GtkWidget *widget,
+		                                                     GdkEventButton *event,
+	                                                             HildonDesktopPopupWindow *popup);
 
 static gboolean popup_grab_on_window (GdkWindow *window, guint32 activate_time, gboolean grab_keyboard);
 
-static gboolean hildon_desktop_popup_window_button_release_event (GtkWidget *widget, GdkEventButton *event);
+static gboolean hildon_desktop_popup_window_button_press_event (GtkWidget *widget, GdkEventButton *event);
 #if defined (MAEMO_CHANGES) && defined(HAVE_XTEST)
 static void hildon_desktop_popup_menu_fake_button_event (GdkEventButton *event, gboolean press);
 #endif
@@ -173,7 +178,7 @@ hildon_desktop_popup_window_class_init (HildonDesktopPopupWindowClass *popup_cla
 
   widget_class->motion_notify_event     = hildon_desktop_popup_window_motion_notify;
   widget_class->leave_notify_event      = hildon_desktop_popup_window_leave_notify;
-  widget_class->button_release_event    = hildon_desktop_popup_window_button_release_event;
+  widget_class->button_press_event    = hildon_desktop_popup_window_button_press_event;
   widget_class->key_press_event 	= hildon_desktop_popup_window_key_press_event;
 	  
   widget_class->realize    = hildon_desktop_popup_window_realize;
@@ -286,9 +291,16 @@ hildon_desktop_popup_window_constructor (GType gtype,
 #else
     popup->priv->extra_panes[i] = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+    gtk_window_set_is_temporary (GTK_WINDOW (popup->priv->extra_panes[i]), TRUE);
+
     gtk_window_set_decorated (GTK_WINDOW (popup->priv->extra_panes[i]), FALSE);
 
     gtk_widget_add_events (GTK_WIDGET (popup), GDK_VISIBILITY_NOTIFY_MASK);
+
+    g_signal_connect (popup->priv->extra_panes[i],
+  		      "delete-event",
+  		      G_CALLBACK (hildon_desktop_popup_window_pane_delete_event),
+  		      NULL);
 #endif
     g_object_ref (G_OBJECT (popup->priv->extra_panes[i]));
     gtk_object_sink (GTK_OBJECT (popup->priv->extra_panes[i]));		 
@@ -297,8 +309,8 @@ hildon_desktop_popup_window_constructor (GType gtype,
 		    	      GDK_WINDOW_TYPE_HINT_MENU);
 
     g_signal_connect (popup->priv->extra_panes[i],
-		      "button-release-event",
-		      G_CALLBACK (hildon_desktop_popup_window_composited_button_release),
+		      "button-press-event",
+		      G_CALLBACK (hildon_desktop_popup_window_composited_button_press),
 		      (gpointer)popup); 
     
     g_signal_connect (popup->priv->extra_panes[i],
@@ -370,7 +382,11 @@ hildon_desktop_popup_window_show (GtkWidget *widget)
 
   for (i=0; i < popup->priv->n_extra_panes; i++)
   {
-    if (popup->priv->extra_panes[i]->requisition.height > 1)	  
+    GtkRequisition req;
+
+    gtk_widget_size_request (popup->priv->extra_panes[i], &req);
+
+    if (req.height > 1)	  
       gtk_widget_show (popup->priv->extra_panes[i]);
   }
 }
@@ -676,6 +692,14 @@ hildon_desktop_popup_window_visibility_notify (GtkWidget          *widget,
   return FALSE;
 }
 
+static gboolean 
+hildon_desktop_popup_window_pane_delete_event (GtkWidget *widget,
+					       GdkEvent *event,
+					       gpointer data)
+{
+  return TRUE;
+}
+
 static gboolean 
 hildon_desktop_popup_window_delete_event (GtkWidget *widget,
 					  GdkEvent *event,
@@ -687,9 +711,9 @@ hildon_desktop_popup_window_delete_event (GtkWidget *widget,
 }
 #endif
 static gboolean
-hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
-                                                       GdkEventButton *event,
-                                                       HildonDesktopPopupWindow *popup)
+hildon_desktop_popup_window_composited_button_press (GtkWidget *widget,
+                                                     GdkEventButton *event,
+                                                     HildonDesktopPopupWindow *popup)
 {
   gboolean in_panes_area  = FALSE,
            in_window_area = FALSE;
@@ -733,8 +757,11 @@ hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
      }
   }
 
+  if (!GTK_WIDGET_VISIBLE (widget))
+    in_panes_area = FALSE; 
+
   /* Event outside of popup or in button area, close in clean way */
-  if (!in_panes_area || in_window_area)
+  if (!in_panes_area && !in_window_area)
   {	  
     hildon_desktop_popup_window_popdown (popup);
 #if defined(MAEMO_CHANGES) && defined(HAVE_XTEST)    
@@ -763,8 +790,8 @@ hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
 }
 
 static gboolean
-hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
-                                                  GdkEventButton *event)
+hildon_desktop_popup_window_button_press_event (GtkWidget *widget,
+                                                GdkEventButton *event)
 {
   HildonDesktopPopupWindow *popup = HILDON_DESKTOP_POPUP_WINDOW (widget);  	
   gboolean in_panes_area  = FALSE,
@@ -784,7 +811,7 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
   else
   {
      for (i=0; i < popup->priv->n_extra_panes; i++)
-     {	     
+     {
        w = popup->priv->extra_panes[i]->allocation.width;
        h = popup->priv->extra_panes[i]->allocation.height;
 
@@ -798,8 +825,8 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
   }
   
   /* Event outside of popup or in button area, close in clean way */
-  if (!in_panes_area || !in_window_area)
-  {	  
+  if (!in_panes_area && !in_window_area)
+  {	 
     hildon_desktop_popup_window_popdown (popup);
 #if defined(MAEMO_CHANGES) && defined(HAVE_XTEST)
     /* This hack sends an extra button-event in order to not lose the event
@@ -816,7 +843,7 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
       
       if ((x < 0) || (x > w) || (y < 0) || (y > h))
       {	      
-        hildon_desktop_popup_menu_fake_button_event (event, TRUE);	      
+        hildon_desktop_popup_menu_fake_button_event (event, TRUE);
         hildon_desktop_popup_menu_fake_button_event (event, FALSE);
       }
     }
@@ -1075,11 +1102,11 @@ hildon_desktop_popup_window_jump_to_pane (HildonDesktopPopupWindow *popup, gint
   {
     if (pane <= -1)
     { 
-      if (popup->priv->have_xgrab)	    
+      if (popup->priv->have_xgrab)
         return;
       else
       { 
-        gtk_grab_remove (popup->priv->pane_with_grab);	      
+        gtk_grab_remove (popup->priv->pane_with_grab);
 
 	popup_grab_on_window (GTK_WIDGET (popup)->window, GDK_CURRENT_TIME, TRUE);
 
