@@ -104,9 +104,6 @@ typedef struct HildonHomeAreaPriv_
 
   GHashTable   *layout;
 
-  GList        *to_add;
-  gboolean      batch_add;
-
   GList        *children_data;
 
   gdouble       default_alpha;
@@ -232,6 +229,9 @@ hildon_home_area_sort_stack (HildonHomeArea *area);
 static void
 hildon_home_area_finalize (GObject *object);
 
+static void
+hildon_home_area_place (HildonHomeArea *area, GtkWidget *widget);
+
 
 
 static void
@@ -826,12 +826,6 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
   g_debug ("Adding Hildon Home applet %s",
            hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (applet)));
 
-  if (priv->batch_add)
-    {
-      priv->to_add = g_list_append (priv->to_add, applet);
-      return;
-    }
-
   if (HILDON_DESKTOP_IS_HOME_ITEM (applet))
     {
       Layout   *layout = NULL;
@@ -846,18 +840,18 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
 
       if (layout)
         {
-          hildon_home_area_put (HILDON_HOME_AREA (area),
-                                applet,
-                                layout->rect.x, layout->rect.y,
-                                layout->stack_index);
           if (layout->rect.width > 0 && layout->rect.height > 0)
             gtk_widget_set_size_request (applet,
                                          layout->rect.width,
                                          layout->rect.height);
+          hildon_home_area_put (HILDON_HOME_AREA (area),
+                                applet,
+                                layout->rect.x, layout->rect.y,
+                                layout->stack_index);
         }
       else
         {
-          hildon_home_area_put (HILDON_HOME_AREA (area), applet, 0, 0, G_MAXINT);
+          hildon_home_area_place (HILDON_HOME_AREA (area), applet);
         }
 
     }
@@ -876,12 +870,12 @@ hildon_home_area_forall (GtkContainer  *container,
 
   i = priv->children_data;
   while (i)
-    {
-      ChildData        *child = i->data;
+  {
+    ChildData        *child = i->data;
 
-      i = i->next;
-      callback (child->widget, user_data);
-    }
+    i = i->next;
+    callback (child->widget, user_data);
+  }
 }
 
 #ifdef HAVE_X_COMPOSITE
@@ -899,6 +893,8 @@ hildon_home_area_child_compose (ChildData *child_data,
 
   alloc = &(child_data->widget->allocation);
 
+  g_return_if_fail (priv->picture != None);
+
   XRenderComposite (GDK_DISPLAY (),
                     (child_data->state == 0)?PictOpOver:PictOpSrc,
                     child_data->picture,
@@ -1646,18 +1642,25 @@ sort_rectangles (GdkRectangle *r1, GdkRectangle *r2)
 }
 
 static void
-hildon_home_area_batch_add (HildonHomeArea *area)
+hildon_home_area_place (HildonHomeArea *area, GtkWidget *w)
 {
   HildonHomeAreaPriv      *priv;
   GdkRectangle         *area_rectangle;
-  GList                *region = NULL, *i;
+  GList                *region = NULL;
+  GtkRequisition    req = {0};
+  GList            *i_rect;
+  const gchar      *name;
+
+  name = hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (w));
+
+  g_debug ("Placing: %s", name);
 
   priv = HILDON_HOME_AREA_GET_PRIVATE (area);
 
   area_rectangle = create_rectangle (0,
                                      0,
                                      GTK_WIDGET (area)->allocation.width,
-                                      GTK_WIDGET (area)->allocation.height);
+                                     GTK_WIDGET (area)->allocation.height);
 
   region = g_list_append (region, area_rectangle);
 
@@ -1666,93 +1669,77 @@ hildon_home_area_batch_add (HildonHomeArea *area)
                          region);
 
   region = g_list_sort (region, (GCompareFunc)sort_rectangles);
-  i = priv->to_add;
 
-  while (i)
+  gtk_widget_size_request (w, &req);
+  g_debug ("size request: %ix%i", req.width, req.height);
+
+  if (req.width > area_rectangle->width ||
+      req.height > area_rectangle->height)
+  {
+    /* Applet is bigger than the area ... Put it in top-left corner */
+    Layout           *layout = g_new (Layout, 1);
+
+    layout->rect.x = 0;
+    layout->rect.y = 0;
+    layout->rect.width = req.width;
+    layout->rect.height = req.height;
+    layout->stack_index = G_MAXINT;
+
+    g_hash_table_insert (priv->layout, g_strdup (name), layout);
+    hildon_home_area_put (area, w, 0, 0, G_MAXINT);
+    g_signal_emit_by_name (area, "layout-changed");
+
+    g_list_foreach (region, (GFunc)g_free, NULL);
+    g_list_free (region);
+
+    return;
+  }
+
+  while (TRUE)
+  {
+    i_rect = region;
+
+    while (i_rect)
     {
-      GtkRequisition    req = {0};
-      GList            *i_rect;
-      GtkWidget        *w;
-      const gchar      *name;
+      GdkRectangle *r = (GdkRectangle *)i_rect->data;
 
-      w = GTK_WIDGET (i->data);
-      name = hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (w));
+      if (r->width  >= req.width &&
+          r->height >= req.height)
+      {
+        Layout           *layout = g_new (Layout, 1);
 
-      gtk_widget_size_request (w, &req);
+        layout->rect.x = r->x;
+        layout->rect.y = r->y;
+        layout->rect.width = req.width;
+        layout->rect.height = req.height;
+        layout->stack_index = G_MAXINT;
 
-      i_rect = region;
+        g_hash_table_insert (priv->layout, g_strdup (name), layout);
+        hildon_home_area_put (area, w, r->x, r->y, G_MAXINT);
+        g_signal_emit_by_name (area, "layout-changed");
 
-      while (i_rect)
-        {
-          GdkRectangle *r = (GdkRectangle *)i_rect->data;
 
-          if (r->width  >= req.width &&
-              r->height >= req.height)
-            {
-              Layout           *layout = g_new (Layout, 1);
-              GdkRectangle     *padded_layout = g_new (GdkRectangle, 1);
+        break;
+      }
 
-              layout->rect.x = r->x;
-              layout->rect.y = r->y;
-              layout->rect.width = req.width;
-              layout->rect.height = req.height;
-              layout->stack_index = G_MAXINT;
-
-              g_hash_table_insert (priv->layout, g_strdup (name), layout);
-              gtk_container_add (GTK_CONTAINER (area), w);
-
-              *padded_layout = layout->rect;
-
-              if (padded_layout->x + padded_layout->width <
-                  area_rectangle->width)
-                padded_layout->width += priv->applet_padding;
-
-              if (padded_layout->y + padded_layout->height <
-                  area_rectangle->height)
-                padded_layout->height += priv->applet_padding;
-
-              if (padded_layout->x)
-                {
-                  padded_layout->x -= priv->applet_padding;
-                  padded_layout->width += priv->applet_padding;
-                }
-
-              if (padded_layout->y)
-                {
-                  padded_layout->y-= priv->applet_padding;
-                  padded_layout->height += priv->applet_padding;
-                }
-
-              substract_rectangle_from_region (region, padded_layout);
-              region = g_list_sort (region, (GCompareFunc)sort_rectangles);
-
-              break;
-            }
-
-          i_rect = g_list_next (i_rect);
-
-        }
-
-      if (!i_rect)
-        {
-          g_debug ("Adding layer");
-          /* Not enough place in this layer, we need to add one */
-          g_list_foreach (region, (GFunc)g_free, NULL);
-          g_list_free (region);
-          region = NULL;
-          region = g_list_append (region, area_rectangle);
-        }
-      else
-        i = g_list_next (i);
+      i_rect = g_list_next (i_rect);
 
     }
 
+    if (!i_rect)
+    {
+      /* Not enough place in this layer, we need to add one */
+      g_list_foreach (region, (GFunc)g_free, NULL);
+      g_list_free (region);
+      region = NULL;
+      region = g_list_append (region, area_rectangle);
+    }
+    else
+      break;
+  }
+
   g_list_foreach (region, (GFunc)g_free, NULL);
   g_list_free (region);
-
-  g_list_free (priv->to_add);
-  priv->to_add = NULL;
-
 }
 
 static void
@@ -1789,6 +1776,17 @@ hildon_home_area_remove (GtkContainer *area, GtkWidget *applet)
 
       child_data_free (child_data);
     }
+
+  if (HILDON_DESKTOP_IS_HOME_ITEM (applet))
+  {
+    const gchar *name =
+        hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (applet));
+
+    if (g_hash_table_remove (priv->layout, name))
+      g_signal_emit_by_name (area, "layout-changed");
+
+  }
+
 }
 
 /* Public functions */
@@ -2074,23 +2072,6 @@ hildon_home_area_get_layout_changed (HildonHomeArea *area)
   return priv->layout_changed;
 }
 
-void
-hildon_home_area_set_batch_add (HildonHomeArea *area, gboolean batch_add)
-{
-  HildonHomeAreaPriv      *priv;
-  g_return_if_fail (area);
-  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
-
-  if (priv->batch_add && !batch_add)
-    {
-      priv->batch_add = FALSE;
-      hildon_home_area_batch_add (area);
-      g_signal_emit_by_name (area, "layout-changed");
-    }
-
-  priv->batch_add = batch_add;
-}
-
 void
 hildon_home_area_put (HildonHomeArea *area,
                       GtkWidget *widget,
