@@ -24,7 +24,6 @@
 #ifndef __LIBMENU_H__
 #define __LIBMENU_H__
 
-#include <libxml/xmlreader.h>
 #include <gtk/gtk.h>
 #include <libintl.h>
 
@@ -78,6 +77,7 @@ enum {
 	TREE_MODEL_DESKTOP_ID,
 	TREE_MODEL_COMMENT,
 	TREE_MODEL_TEXT_DOMAIN,
+	TREE_MODEL_DEFAULT,
 	TREE_MODEL_COLUMNS
 };
 
