@@ -170,7 +170,7 @@ static void
 hd_keys_launch_application (HDKeysConfig *keys,
 			    gpointer     *user_data)
 {
-  hd_wm_activate_service ("isearch-applet.desktop", NULL);
+  hd_wm_activate_service ("com.nokia.osso_global_search", NULL);
 }
 
 static void 
@@ -257,14 +257,11 @@ HDKeysActionConfLookup[] =
     hd_keys_switch_window, GINT_TO_POINTER (TRUE) },
   { HD_KEYS_GCONF_PATH "/application_close",    HD_KEY_ACTION_APPLICATION_CLOSE,
     hd_keys_action_application_close, NULL},
-  { HD_KEYS_GCONF_PATH "/global/isearch_applet", HD_KEY_ACTION_APPLICATION,
+  { HD_KEYS_GCONF_PATH "/global/osso-global-search", HD_KEY_ACTION_APPLICATION,
     hd_keys_launch_application, NULL },
   { NULL, 0, NULL, NULL }
 };
 
-
-     	
-
 static gboolean
 hd_keys_keysym_needs_shift (KeySym keysym)
 {
