@@ -592,7 +592,7 @@ hd_wm_window_process_net_wm_icon (HDWMWindow *win)
 
       hd_wm_debug("got w:%i, h:%im offset is %i\n", w, h, offset);
 
-      if (w == 26 && h == 26)
+      if (w == 40 && h == 40)
 	break;
 
       offset += ((w*h) + 2);
@@ -646,7 +646,7 @@ out:
     XFree (data);
 }
 
-	static void
+static void
 hd_wm_window_process_wm_name (HDWMWindow *win)
 {
   int                    n_items = 0;
@@ -688,9 +688,8 @@ hd_wm_window_process_wm_name (HDWMWindow *win)
                          8,
                          0,
                          NULL);
-  
-  if (win->priv->info)
-    g_signal_emit_by_name (hdwm,"entry_info_changed",win->priv->info);
+
+  g_signal_emit_by_name (hdwm,"entry_info_changed", HD_WM_ENTRY_INFO (win));
 }  
 
 static void
