@@ -739,7 +739,7 @@ mce_handler (DBusConnection *conn,
     g_signal_emit_by_name (hdwm, "long-key-press");
 #ifdef MAEMO_CHANGES    
     gdk_close_all_temporary_windows ();
-#endif    
+#endif   
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
   }
   
@@ -1994,6 +1994,7 @@ hd_wm_process_x_client_list (HDWM *hdwm)
       if (wm_type_atom[0] == hdwm->priv->atoms[HD_ATOM_NET_WM_WINDOW_TYPE_DESKTOP])
       {
 	hd_wm_desktop_set_x_window (HD_WM_DESKTOP (hd_wm_get_home_info (hdwm)), allwins.wins[i]);
+        XFree (wm_type_atom);
 	break;
       }	      
 
@@ -2319,8 +2320,7 @@ hd_wm_activate_window (guint32 what, GdkWindow *window)
       hd_wm_focus_active_window (hdwm);
       return;
     case HD_TN_ACTIVATE_ALL_MENU:
-      g_object_set (hdwm->priv->all_menu, "activate", TRUE, NULL);
-      g_signal_emit_by_name (hdwm->priv->all_menu, "toggled");
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (hdwm->priv->all_menu), TRUE);
       break;
     default:
       hd_wm_debug ("%s: %d, hd_wm_activate: deprecated. It was used to activate specific tasknavigator buttons",__FILE__,__LINE__);
@@ -3283,3 +3283,52 @@ hd_wm_switch_instance_current_window (HDWM *hdwm, gboolean to_next)
      to_next);
 }	
 
+void
+hd_wm_switch_instance_current_application (HDWM *hdwm, gboolean to_next)
+{
+  GList *applications_list = NULL, *iter, *new_entry;
+  HDWMApplication *application;
+  HDWMWindow *window;
+
+  if (!hdwm->priv->active_window)
+    return;
+
+  applications_list = hd_wm_get_applications (hdwm);
+
+  for (iter = applications_list; iter != NULL; iter = g_list_next (iter))
+  {
+    application = HD_WM_APPLICATION (iter->data);
+
+    window = hd_wm_application_get_active_window (application);
+
+    if (window == hdwm->priv->active_window)
+    {
+      if (to_next)
+      {
+        new_entry = g_list_next (iter);
+
+        if (!new_entry)
+          new_entry = g_list_first (applications_list);
+      }
+      else
+      {
+        new_entry = g_list_previous (iter);
+
+        if (!new_entry)
+          new_entry = g_list_last (applications_list);
+      }
+
+      if (new_entry == iter) /* No need to emit any signal */
+        return;	      
+      
+      HDWMWindow *next_win = 
+        hd_wm_application_get_active_window (HD_WM_APPLICATION (new_entry->data));
+
+      HDWMEntryInfo *info = HD_WM_ENTRY_INFO (next_win);
+
+      hd_wm_top_item (info);
+      g_signal_emit_by_name (hdwm, "entry_info_stack_changed", info);
+      break;
+    }
+  }
+}
