@@ -510,35 +510,6 @@ hd_applications_menu_activate_app (GtkMenuItem *item, HDApplicationsMenu *button
     
         g_clear_error (&error);
       }
-      else
-      {
-        int priority;
-        errno = 0;
-        gchar *oom_filename;
-        int fd;
-
-        /* If the child process inherited desktop's high priority,
-         * give child default priority */
-        priority = getpriority (PRIO_PROCESS, child_pid);
-    
-        if (!errno && priority < 0)
-        {
-          setpriority (PRIO_PROCESS, child_pid, 0);
-        }
-
-        /* Unprotect from OOM */
-        oom_filename = g_strdup_printf ("/proc/%i/oom_adj",
-                                        child_pid);
-        fd = open (oom_filename, O_WRONLY);
-        g_free (oom_filename);
-
-        if (fd >= 0)
-          {
-            write (fd, "0", sizeof (char));
-            close (fd);
-          }
-
-      }
     }
     else
     {
@@ -1169,8 +1140,9 @@ hd_applications_menu_button_toggled (GtkWidget *widget, HDApplicationsMenu *butt
     selected_item = 
       hildon_desktop_popup_menu_get_selected_item (button->priv->menu_applications);
 
-    hildon_desktop_popup_menu_deselect_item (button->priv->menu_applications,
-		    			     selected_item);
+    if (GTK_IS_MENU_ITEM (selected_item))
+      hildon_desktop_popup_menu_deselect_item (button->priv->menu_applications,
+  		    			       selected_item);
 
     hildon_desktop_popup_window_popdown (button->priv->popup_window);
 
