@@ -25,6 +25,8 @@
 #include "hd-home-background.h"
 #include "background-manager/hildon-background-manager.h"
 
+#include <libhildondesktop/hildon-desktop-picture.h>
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -401,13 +403,14 @@ hd_home_background_apply (HildonDesktopBackground *background,
   DBusGProxy           *background_manager_proxy;
   DBusGConnection      *connection;
   GError               *local_error = NULL;
-  gint                  pixmap_xid;
   gint32                top_offset, bottom_offset, right_offset, left_offset;
   gchar                *filename;
   gchar                *cache;
   gchar                *new_cache = NULL;
   gchar                *file_to_use;
+  guint                 width, height;
   GdkColor             *color;
+  Picture               picture;
   HildonDesktopBackgroundMode   mode;
 
   g_return_if_fail (HD_IS_HOME_BACKGROUND (background) && window);
@@ -442,6 +445,10 @@ hd_home_background_apply (HildonDesktopBackground *background,
     file_to_use = filename;
   }
 
+  gdk_drawable_get_size (window, &width, &height);
+  picture = hildon_desktop_picture_from_color (color, width, height);
+  g_return_if_fail (picture != None);
+
   background_manager_proxy =
       dbus_g_proxy_new_for_name (connection,
                                  HILDON_BACKGROUND_MANAGER_SERVICE,
@@ -451,17 +458,18 @@ hd_home_background_apply (HildonDesktopBackground *background,
   top_offset = bottom_offset = right_offset = left_offset = 0;
 #define S(string) (string?string:"")
   org_maemo_hildon_background_manager_set_background (background_manager_proxy,
-                                                      GDK_WINDOW_XID (window),
+                                                      picture,
+                                                      width,
+                                                      height,
                                                       S(file_to_use),
                                                       S(cache),
-                                                      color->red,
-                                                      color->green,
-                                                      color->blue,
                                                       mode,
-                                                      &pixmap_xid,
                                                       error);
 #undef S
 
+  if (error)
+      XRenderFreePicture (GDK_DISPLAY (), picture);
+
   if (new_cache && !error)
     g_object_set (background,
                   "cache", new_cache,
@@ -478,29 +486,25 @@ struct cb_data
   gpointer                              user_data;
   GdkWindow                            *window;
   gchar                                *new_cache;
+  Picture                               picture;
 };
 
 
 static void
 hd_home_background_apply_async_dbus_callback (DBusGProxy       *proxy,
-                                              gint              picture_id,
                                               GError           *error,
                                               struct cb_data   *data)
 {
   if (data->background->priv->cancelled)
     {
+      XRenderFreePicture (GDK_DISPLAY (), data->picture);
       g_free (data);
       return;
     }
 
   if (error)
     {
-      goto cleanup;
-    }
-
-  if (!picture_id)
-    {
-      g_warning ("No picture id returned");
+      XRenderFreePicture (GDK_DISPLAY (), data->picture);
       goto cleanup;
     }
 
@@ -513,7 +517,7 @@ cleanup:
 
   if (data->callback)
     data->callback (HILDON_DESKTOP_BACKGROUND (data->background),
-                    picture_id,
+                    data->picture,
                     error,
                     data->user_data);
 
@@ -540,6 +544,7 @@ hd_home_background_apply_async (HildonDesktopBackground        *background,
   gchar                    *new_cache = NULL;
   gchar                    *file_to_use;
   GdkColor                 *color;
+  guint                     width, height;
   HildonDesktopBackgroundMode   mode;
 
   g_return_if_fail (HD_IS_HOME_BACKGROUND (background) && window);
@@ -592,6 +597,8 @@ hd_home_background_apply_async (HildonDesktopBackground        *background,
     file_to_use = filename;
   }
 
+  gdk_drawable_get_size (window, &width, &height);
+
   data = g_new (struct cb_data, 1);
 
   data->callback = cb;
@@ -599,6 +606,9 @@ hd_home_background_apply_async (HildonDesktopBackground        *background,
   data->user_data = user_data;
   data->window = window;
   data->new_cache = new_cache;
+  data->picture = hildon_desktop_picture_from_color (color, width, height);
+
+  g_return_if_fail (data->picture != None);
 
   priv->cancelled = FALSE;
 
@@ -606,12 +616,11 @@ hd_home_background_apply_async (HildonDesktopBackground        *background,
 #define S(string) (string?string:"")
   org_maemo_hildon_background_manager_set_background_async
                                                 (background_manager_proxy,
-                                                 GDK_WINDOW_XID (window),
+                                                 data->picture,
+                                                 width,
+                                                 height,
                                                  S(file_to_use),
                                                  S(cache),
-                                                 color->red,
-                                                 color->green,
-                                                 color->blue,
                                                  mode,
                                                  (org_maemo_hildon_background_manager_set_background_reply) hd_home_background_apply_async_dbus_callback,
                                                  data);
