@@ -745,7 +745,8 @@ hd_home_window_root_event_filter (GdkXEvent *xevent,
                                    &format, &n_res,
                                    &extra, (unsigned char **) &res.pointer);
 
-      gdk_error_trap_pop ();
+      if (gdk_error_trap_pop ())
+        return GDK_FILTER_CONTINUE;
 
       if ((status != Success) || (real_type != XA_WINDOW) ||
           (format != 32) || (n_res != 1) || (res.win == NULL) ||
@@ -801,6 +802,7 @@ hd_home_window_expose (GtkWidget *widget, GdkEventExpose *event)
                                               &x_offset, &y_offset);
 
           picture = hildon_desktop_picture_from_drawable (drawable);
+          g_return_val_if_fail (picture != None, FALSE);
 
           XRenderComposite (GDK_DISPLAY (),
                             PictOpSrc,
@@ -811,7 +813,6 @@ hd_home_window_expose (GtkWidget *widget, GdkEventExpose *event)
                             0, 0,
                             - x_offset, - y_offset,
                             widget->allocation.width, widget->allocation.height);
-
           XRenderFreePicture (GDK_DISPLAY (), picture);
           XSync (GDK_DISPLAY (), FALSE);
         }
@@ -1232,7 +1233,8 @@ hd_home_window_area_remove (HDHomeWindow *window)
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
   area = GTK_BIN (window)->child;
 
-  hd_home_window_save_area_layout (window);
+  if (!priv->selecting_applets)
+    g_signal_emit_by_name (window, "save");
 
   hd_home_window_ensure_menu_status (window);
 }
@@ -1463,9 +1465,7 @@ hd_home_window_select_applets (HDHomeWindow *window)
 
   priv->selecting_applets = TRUE;
 
-  hildon_home_area_set_batch_add (HILDON_HOME_AREA (area), TRUE);
   g_signal_emit_by_name (window, "select-plugins", NULL);
-  hildon_home_area_set_batch_add (HILDON_HOME_AREA (area), FALSE);
   priv->selecting_applets = FALSE;
 
 }
