@@ -167,6 +167,7 @@ hd_select_plugins_dialog_get_store (GList *loaded_plugins,
   {
     gchar *desktop_path = NULL;
     gchar *name = NULL;
+    gchar *text_domain = NULL;
     GList *active;
     error = NULL;
 
@@ -193,11 +194,10 @@ hd_select_plugins_dialog_get_store (GList *loaded_plugins,
       return NULL;
     }
 
-    name = g_key_file_get_locale_string (keyfile,
-                                         HD_PLUGIN_CONFIG_GROUP,
-                                         HD_PLUGIN_CONFIG_KEY_NAME,
-                                         NULL /* current locale */,
-                                         &error);
+    name = g_key_file_get_string (keyfile,
+                                  HD_PLUGIN_CONFIG_GROUP,
+                                  HD_PLUGIN_CONFIG_KEY_NAME,
+                                  &error);
 
     if (error)
     {
@@ -217,12 +217,18 @@ hd_select_plugins_dialog_get_store (GList *loaded_plugins,
 
     gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 
+    text_domain = g_key_file_get_string (keyfile, 
+		                         HD_PLUGIN_CONFIG_GROUP,
+				         HD_PLUGIN_CONFIG_KEY_TEXT_DOMAIN, 
+					 NULL);
+
     gtk_list_store_set (GTK_LIST_STORE (store), &iter, 
-                        HD_SPD_COLUMN_NAME, _(name),
+                        HD_SPD_COLUMN_NAME, (text_domain ? dgettext(text_domain, name) : _(name)),
                         HD_SPD_COLUMN_ACTIVE, active,
                         HD_SPD_COLUMN_DESKTOP_FILE, desktop_path,
                         -1);
 
+    g_free(text_domain);
     g_free (name);
   }
 
