@@ -279,24 +279,25 @@ hd_switcher_menu_item_icon_animation (GtkWidget *icon,
   if (is_on)
   {
     pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (icon));
-              
+
     pixbuf_anim = hn_app_pixbuf_anim_blinker_new (pixbuf,
                                                   1000 / ANIM_FPS,
 					          -1,
 						  10);
 
-    gtk_image_set_from_animation (GTK_IMAGE(icon), pixbuf_anim);
+    gtk_image_set_from_animation (GTK_IMAGE (icon), pixbuf_anim);
 
     g_object_unref (pixbuf_anim);
   }
-  else
+  else if (gtk_image_get_storage_type (GTK_IMAGE (icon)) == GTK_IMAGE_ANIMATION)
   {
     pixbuf_anim = gtk_image_get_animation (GTK_IMAGE (icon));
-    g_debug ("Turning animation off");      
-    /* grab static image from menu item and reset */
+
+    /* Grab static image from menu item and reset */
     pixbuf = gdk_pixbuf_animation_get_static_image (pixbuf_anim);
 
     gtk_image_set_from_pixbuf (GTK_IMAGE (icon), pixbuf);
+
     g_object_unref (pixbuf);
   }
 }
