@@ -64,6 +64,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
+#undef DEBUG
 
 /* Menu item strings */
 #define AS_HOME_ITEM 		_("tana_fi_home")
@@ -410,12 +411,13 @@ hn_app_button_create_menu (HNAppButton *app_button)
 
   menu = gtk_menu_new ();
 
-/*#ifndef MAEMO_CHANGES*/
+#ifndef MAEMO_CHANGES
  /* If we attach it doesn't show up with VKB */
   gtk_menu_attach_to_widget (GTK_MENU (menu), 
 		  	     GTK_WIDGET (app_button),
 			     NULL);
-/*#endif  */
+#endif
+
   children = hd_wm_entry_info_get_children (info);
   for (l = children; l != NULL; l = l->next)
   {
@@ -475,8 +477,10 @@ menu_unmap_cb (GtkWidget   *widget,
   if (app_button->priv->prev_button &&
       app_button->priv->prev_button != GTK_TOGGLE_BUTTON(app_button))
     {
+#ifdef DEBUG
       g_debug ("Retoggling previously toggled button");
-	 
+#endif	 
+      
       /*gtk_toggle_button_set_active (app_button->priv->prev_button, TRUE);*/
       gtk_toggle_button_toggled (app_button->priv->prev_button);
     }
@@ -505,6 +509,7 @@ menu_button_release_cb (GtkWidget      *widget,
    */
   gtk_widget_get_pointer(widget, &x, &y);
 
+#ifdef DEBUG
   g_debug ("pointer [%d,%d],\n"
            "allocation [%d, %d, %d, %d]",
           x, y,
@@ -512,6 +517,7 @@ menu_button_release_cb (GtkWidget      *widget,
           widget->allocation.y,
           widget->allocation.width,
           widget->allocation.height);
+#endif
 
   if(x > widget->allocation.width  || x < 0 ||
      y > widget->allocation.height || y < 0)
@@ -521,13 +527,15 @@ menu_button_release_cb (GtkWidget      *widget,
 
       if (app_button->priv->prev_button)
         {
+#ifdef DEBUG
           g_debug ("Retoggling previously toggled button");
-	 
+#endif
           gtk_toggle_button_set_active (app_button->priv->prev_button, TRUE);
           gtk_toggle_button_toggled (app_button->priv->prev_button);
         }
-
+#ifdef DEBUG
       g_debug ("%s: %d, hd_wm_activate(HN_TN_ACTIVATE_LAST_APP_WINDOW);",__FILE__,__LINE__);
+#endif
       return TRUE;
     }
   
@@ -537,10 +545,24 @@ menu_button_release_cb (GtkWidget      *widget,
 static gboolean
 menu_keypress_cb (GtkWidget *menu, GdkEventKey *event, HNAppButton * app_button)
 {
+  gboolean cancel_menu = FALSE;
+
   g_return_val_if_fail(event, FALSE);
 
-  if (event->keyval == GDK_Left ||
-      event->keyval == GDK_KP_Left ||
+  if (gtk_widget_get_direction (menu) == GTK_TEXT_DIR_RTL)
+  {
+    if (event->keyval == GDK_Right || 
+             event->keyval == GDK_KP_Right)
+      cancel_menu = TRUE;
+  } 
+  else 
+  {
+    if (event->keyval == GDK_Left || 
+             event->keyval == GDK_KP_Left)
+      cancel_menu = TRUE;
+  }  
+
+  if (cancel_menu ||
       event->keyval == GDK_Escape)
     {
       gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu));
@@ -555,13 +577,15 @@ menu_keypress_cb (GtkWidget *menu, GdkEventKey *event, HNAppButton * app_button)
           /* pointer outside the button, i.e., press canceled */
           if (app_button->priv->prev_button)
             {
+#ifdef DEBUG
               g_debug ("Retoggling previously toggled button");
-	 
+#endif
               gtk_toggle_button_set_active (app_button->priv->prev_button, TRUE);
               gtk_toggle_button_toggled (app_button->priv->prev_button);
             }
-
+#ifdef DEBUG
 		  g_debug ("%s: %d, hd_wm_activate(HN_TN_ACTIVATE_KEY_FOCUS);",__FILE__,__LINE__);
+#endif
 		  gtk_widget_grab_focus (GTK_WIDGET (app_button));
         }
       
@@ -674,7 +698,9 @@ hn_app_button_key_press_event (GtkWidget * widget,
 
   if (!event)
     {
+#ifdef DEBUG
       g_debug ("no event given !!!");
+#endif
       return FALSE;
     }
   
@@ -729,13 +755,16 @@ hn_app_button_release_event (GtkWidget      *widget,
   HNAppButtonPrivate *priv = app_button->priv;
   gint x,y;
   gboolean untoggle = FALSE;
-  
+
+#ifdef DEBUG
   g_debug("Button released ...");
+#endif
 
   if (priv->tooltip)
     {
+#ifdef DEBUG
       g_debug("removing tooltip timer");
-      
+#endif
       /* have to remove the show timer, so that if the tooltip is not
        * yet showing, it does not get displayed
        */
@@ -1020,8 +1049,6 @@ hn_app_button_set_group (HNAppButton *app_button,
   if (g_slist_find (group, app_button))
     return;
 
-  g_debug ("setting group for the button");
-
   if (app_button->group)
     {
       GSList *l;
@@ -1401,9 +1428,11 @@ void
 hn_app_button_set_is_blinking (HNAppButton *button,
 			       gboolean     is_blinking)
 {
+#ifdef DEBUG
   g_debug("Setting is_blinking %d -> %d",
          button->priv->is_blinking,
          is_blinking);
+#endif
   
   g_return_if_fail (HN_IS_APP_BUTTON (button));
 
