@@ -29,23 +29,85 @@
 #include <locale.h>
 
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
 
+/* getpriority, setpriority */
+#include <sys/time.h>
+#include <sys/resource.h>
+
+/* dlsym */
+#define __USE_GNU
+#include <dlfcn.h>
+
+/* open */
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
+#include <errno.h>
+
 #include "hd-desktop.h"
 
 #define OSSO_USER_DIR        ".osso"
 #define HILDON_DESKTOP_GTKRC "current-gtk-theme.maemo_af_desktop"
 
+/*
+ * hildon-desktop is typically running with higher priority and
+ * runs protected against out-of-memory killing from the kernel.
+ * Spawned children however should not be given these priviledges.
+ */
+pid_t
+fork (void)
+{
+  static pid_t (*fork_real) () = NULL;
+  pid_t         pid;
+
+  if (fork_real == NULL)
+    fork_real = dlsym (RTLD_NEXT, "fork");
+
+  pid = fork_real ();
+
+  if (pid == 0)
+  {
+    pid_t       child_pid;
+    int         priority;
+    int         fd;
+
+    errno = 0;
+
+    child_pid = getpid ();
+
+    /* If the child process inherited desktop's high priority,
+     * give child default priority */
+    priority = getpriority (PRIO_PROCESS, child_pid);
+
+    if (!errno && priority < 0)
+    {
+      setpriority (PRIO_PROCESS, child_pid, 0);
+    }
+
+    /* Unprotect from OOM */
+    fd = g_open ("/proc/self/oom_adj", O_WRONLY, 0);
+
+    if (fd >= 0)
+    {
+      write (fd, "0", sizeof (char));
+      close (fd);
+    }
+
+  }
+
+  return pid;
+}
+
 int main (int argc, char **argv)
 {
   GObject *desktop;
   gchar *gtkrc = NULL;
 
   g_thread_init (NULL);
-  gdk_threads_init ();
-  gdk_threads_enter ();
-
   setlocale (LC_ALL, "");
 
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -85,7 +147,6 @@ int main (int argc, char **argv)
   hd_desktop_run (HD_DESKTOP (desktop));
 
   gtk_main ();
-  gdk_threads_leave ();
 
   return 0;
 }
